unit DtEdit97;

(******************************************************************************
TDateEdit97

Derived from
    tCustomEdit

Properties
  Not published
    DateCorrect      : boolean read FDateCorrect;
    DayArray         : array which contains all days names
    MonthArray       : array which contains all monthes names
    ValidDates       : List of valid dates (read only)
  Published
    Align,
    BorderStyle,
    Color,
    Enabled,
    Font,
    ParentFont,
    TabOrder,
    TabStop          : inherited from tCustomEdit
    BevelInner,
    BevelOuter       : properties for popup calendar
    ButtonDateStyle  : style of popup button (drop down or calendar)
    ButtonStyle      : select button style (new, win31, autodetect)
    CenturyThreshold : value when a date is set to next century (when UseThreshold is true)
    ColorCalendar    : Colors for the popup calendar (title, arrows, dates, ...)
      ColorArrow     : Color for arrow buttons of calendar
      ColorDates     : Color for dates of calendar
      ColorTitle     : Color for title of calendar
      ColorValid     : Color for valid dates of calendar
      ColorWeekEnd   : Color for week-end dates of calendar
    ColorError       : Color when date isn't correct
    Date             : Selected date as DateTime
    DayNames         : Names of each days
    KeyRepeat        : Delay when a key is repeated
    MonthNames       : Names of each monthes
    OnlyMonth        : If set to true, can select month without days.
    OnlyValidDates   : If set to true, only valid dates are allowed. Others are grayed
    Options          : Misc. options for DateEdit component
      ButtonTabStop  : choose if buttons are TabStop
      CanClear       : show (or hide) clear button
      CanPopup       : enable (or disable) popup calendar button
      CanValidate    : Show (or hide) validate button
      EmptyAsToday   : If true (and canclear is false), empty date is "today"
      FocusOnError   : If date is invalid, keep focus or not
      IsMasked       : choose if you want to use mask-edit for input
      IsModal        : choose if you want a modal popup calendar (or not)
      NumWeek        : If true, display week number on popup calendar
      PopupToday     : If true, popup calendar always start with today's date
      ShowToday      : Show today button in the calendar.
      ShowCancel     : Show cancel button in the calendar.
    ShortCuts        : ShortCuts for buttons (Clear, Popupand Validate)
      BtnClear       : Shortcut key for Clear button
      BtnPopup       : Shortcut key for Popup button
      BtnValidate    : Shortcut key for Validate button
    StartOfWeek      : First day of week
    StartOfWeekEnd   : First day of week-end: saturday, or thursday for musulmans
    Text             : Selected date as String
    UseThreshold     : Use CenturyThreshold value for assign a date to next centure (or not)
    WeekEndDays      : Number of days in a week-end: 1 or 2, or more if you want!

procedures
    AddValidDate     : Add a ValidDate
    DateInList       : Check if a date is in ValidDates list
    ClearValidDates  : Clear ValidDates list
    ClearDate        : Clear date
    PopupCalendar    : Popup the calendar
    SetDateCorrect   : Set if entered date is correct (change drawing color)
    AfterClick       : function calling OnAfterClick event;
    ValidateDate     : Function calling OnValidateClick event.

events
    OnChange,
    OnEnter,
    OnExit,
    OnKeyDown,
    OnKeyPress,
    OnKeyUp          : inherited from tCustomEdit
    OnAfterClick     : When a user click for select a date.
    OnValidateClick  : Used for validate an entered date.
    AfterNewMonth    : After changing month
    BeforeNewMonth   : Before changing month


Author name=BOURMAD Mehdi
Author E-mail=bourmad@multimania.com
Author URL=www.multimania.com/bourmad
******************************************************************************)

interface

uses
  Clipbrd,
  Dialogs,

  SysUtils, WinTypes, WinProcs, Messages, Buttons, Classes, Controls,
  DsgnIntf, ExtCtrls, Forms, Graphics, Menus, StdCtrls,
  DtMisc97, DtFunc97, DtPop97;

type
{******************** DateEdit97 Component *************************}
  TDateEdit97 = class(TCustomEdit)
  private
    FAbout: TAboutMeProperty;
    FAlignment: TAlignment;
    FBevelInner: tPanelBevel;
    FBevelOuter: tPanelBevel;
    FBtnDateStyle: tDateBtnStyle;
    FBtnStyle: tButtonStyle;
    FCanvas: TControlCanvas;
    FCenturyThreshold: byte;
    FColorCalendar: TCalendarColors;
    FColorError,
    FColorNormal: TColor;
    FDate: TDateTime;
    FDateCorrect: boolean;
    FDayName: TDayName;
    FKeyRepeat: Word;
    FDayArray: TDayArray;
    FMonthArray: TMonthArray;
    FMonthName: TMonthName;
    FOnlyMonth: Boolean;
    FOnlyValidDates: Boolean;
    FStartOfWeek,
    FStartOfWeekEnd: TDayOfWeek;
    FWeekEndDays: Byte;
    FNewShortDateFormat: string;
    FOnAfterClick,
    FOnValidated: TNotifyEvent;
    FOptions: tDateOptions;
    FShortCuts: TButtonsShortCut;
    FUseThreshold: Boolean;
    FrmCalendar: tDtCalPop97;
    FBeforeNewMonth: TNewMonthEvent;
    FAfterNewMonth: TNewMonthEvent;
  protected
    FBtnPopup,
    FBtnValidate,
    FBtnClear: tBitBtn;
    Procedure ClosePopFct (Sender: TObject; var Action: TCloseAction);
    procedure CreateFormCalendar;
    procedure InitFormCalendar;
    function RemoveChr (aChar: Char; aStr: String): String;
    function PadDate (aStr: String): String;
    function InterPretDate (TextDate: string): string;
{    procedure NumToDate;}
    function NumToDate : String;
    procedure DateToNum;
    procedure DayNameChange(ADayName : TObject);
    procedure MonthNameChange(AMonthName : TObject);
    function GetDayArray: PDayArray;
    function GetMonthArray: PMonthArray;
    function GetValidDates: tStringList;
    function GetTextMargins: TPoint;
    procedure SetAlignment(value: TAlignment);
    procedure SetBevelInner(Value: TPanelBevel);
    procedure SetBevelOuter(Value: TPanelBevel);
    procedure SetBtnDateStyle (value: tDateBtnStyle);
    procedure SetBtnStyle (value: tButtonStyle);
    procedure SetDate(value: TDateTime);
    procedure SetTextFromDate;
    procedure SetKeyRepeat (value: word);
    procedure SetOnlyMonth (value: boolean);
    procedure SetOnlyValidDates (value: boolean);
    procedure SetOptions (value: tDateOptions);
    procedure SetStartOfWeek(Value: TDayOfWeek);
    procedure SetStartOfWeekEnd(Value: TDayOfWeek);
    procedure SetWeekEndDays (value: byte);
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure WMSize(var message: TWMSize); message WM_SIZE;
    procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
    procedure KeyPress(var Key: char); override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure DoEnter; override;
    procedure DoExit; override;
    procedure FireBeforeNewMonthEvent(Sender: TObject; FromDate, TillDate: TDateTime);
    procedure FireAfterNewMonthEvent(Sender: TObject; FromDate, TillDate: TDateTime);
  public
    procedure Loaded; override;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure CreateParams(var Params: TCreateParams); override;
    function DateInList(dt: TDateTime): Boolean;
    procedure AddValidDate(dt: TDateTime);
    procedure ClearValidDates;
    procedure ClearDate(Sender: TObject);
    procedure PopupCalendar(Sender: TObject);
    procedure SetDateCorrect(pValue: boolean);
    procedure AfterClick(Sender: TObject);
    procedure ValidateDate(Sender: TObject);
    procedure DoDate;
{    procedure DoEdit;}
    {properties}
    property DateCorrect: boolean read FDateCorrect;
    property DayArray: PDayArray read GetDayArray;
    property MonthArray: PMonthArray read GetMonthArray;
    property ValidDates: tStringList read GetValidDates;
  published
    {properties}
    property Align;
    property BorderStyle;
    property Color;
    property Enabled;
    property Font;
    property ParentFont;
    property TabOrder;
    property TabStop;
    property About: TAboutMeProperty read FAbout write FAbout;
    {actually, Alignment doesn't works correctly}
    property Alignment: TAlignment read FAlignment write SetAlignment default taLeftJustify;
    property BevelInner: tPanelBevel read FBevelInner write SetBevelInner default bvNone;
    property BevelOuter: tPanelBevel read FBevelOuter write SetBevelOuter default bvRaised;
    property ButtonDateStyle: tDateBtnStyle read FBtnDateStyle write SetBtnDateStyle default btsCalendar;
    property ButtonStyle: tButtonstyle read FBtnStyle write SetBtnStyle default bsAutodetect;
    property CenturyThreshold: byte read FCenturyThreshold write FCenturyThreshold default 50;
    property ColorCalendar: tCalendarColors read FColorCalendar write FColorCalendar;
    property ColorError: tColor read FColorError write FColorError default clRed;
    property Date: TDateTime read FDate write SetDate;
    property DayNames: TDayName read FDayName write FDayName;
    property KeyRepeat: Word read FKeyRepeat write SetKeyRepeat default 100;
    property MonthNames: TMonthName read FMonthName write FMonthName;
    property OnlyMonth: boolean read FOnlyMonth write SetOnlyMonth default false;
    property OnlyValidDates: boolean read FOnlyValidDates write SetOnlyValidDates default false;
    property Options: tDateOptions read FOptions write SetOptions;
    property ShortCuts: TButtonsShortCut read FShortCuts write FShortCuts;
    property StartOfWeek: TDayOfWeek read FStartOfWeek write SetStartOfWeek default 1;
    property StartOfWeekend: TDayOfWeek read FStartOfWeekEnd write SetStartOfWeekEnd default 6;
    property UseThreshold: Boolean read FUseThreshold write FUseThreshold default false;
    property WeekEndDays: byte read FWeekEndDays write SetWeekEndDays default 2;
    {events}
    property OnChange;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnAfterClick: TNotifyEvent read FOnAfterClick write FOnAfterClick;
    property OnValidateClick: TNotifyEvent read FOnValidated write FOnValidated;
    property AfterNewMonth: TNewMonthEvent read FAfterNewMonth write FAfterNewMonth;
    property BeforeNewMonth: TNewMonthEvent read FBeforeNewMonth write FBeforeNewMonth;
  end;

implementation

{$IFNDEF WIN32}
  {$R *.R16}
{$ELSE}
  {$R *.R32}
{$ENDIF}

{-------------------------------------------------------------}
{---------------------- TDateEdit97 --------------------------}
{-------------------------------------------------------------}
constructor TDateEdit97.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
{BM}  Parent := tWinControl (aOwner);{}
  FrmCalendar := nil;

  ControlStyle := ControlStyle - [csSetCaption]
                 {$IFDEF WIN32}+ [csReplicatable]{$ENDIF};

  FAlignment := taLeftJustify;
  FBtnDateStyle := btsCalendar;
  FBtnStyle := bsAutodetect;
  FOptions := [doButtonTabStop, doCanClear, doCanPopup,
               doIsMasked, doShowCancel, doShowToday];
  FColorError := clRed;
  FBevelInner := bvNone;
  FBevelOuter := bvRaised;
  FKeyRepeat := 100;
  FOnlyMonth := false;
  FOnlyValidDates := false;
  FStartOfWeek := 1;
  FStartOfWeekend := 6;
  FUseThreshold := false;
  FCenturyThreshold := 50;
  FWeekEndDays := 2;

  FNewShortDateFormat := PadDate (RemoveChr (DateSeparator, ShortDateFormat));

  FColorCalendar := TCalendarColors.Create;
  FDayName := TDayName.Create;
  FDayName.OnChange := DayNameChange;
  FMonthName := TMonthName.Create;
  FMonthName.OnChange := MonthNameChange;
  FShortCuts := TButtonsShortCut.Create;

  SetDate(0);
  SetDateCorrect(True);

  if doCanClear in FOptions
  then begin
    FBtnClear := tBitBtn.Create (Self);
    FBtnClear.Visible := True;
    FBtnClear.Parent := Self;
    DeleteObject (FBtnClear.Glyph.Handle);
    FBtnClear.Glyph.ReleaseHandle;
    FBtnClear.Glyph.Handle := LoadBitmap(hInstance, 'BTNCLEAR');
    FBtnClear.NumGlyphs := 2;
    FBtnClear.OnClick := ClearDate;
    FBtnClear.TabStop := doButtonTabStop in FOptions;
  end;

  if doCanValidate in FOptions
  then begin
    FBtnValidate := tBitBtn.Create (Self);
    FBtnValidate.Visible := True;
    FBtnValidate.Parent := Self;
    DeleteObject (FBtnValidate.Glyph.Handle);
    FBtnValidate.Glyph.ReleaseHandle;
    FBtnValidate.Glyph.Handle := LoadBitmap(hInstance, 'BTNVALIDATE');
    FBtnValidate.NumGlyphs := 2;
    FBtnValidate.OnClick := ValidateDate;
    FBtnValidate.TabStop := doButtonTabStop in FOptions;
  end;

  FBtnPopup := tBitBtn.Create (Self);
  FBtnPopup.Visible := True;
  FBtnPopup.Parent := Self;
  DeleteObject (FBtnPopup.Glyph.Handle);
  FBtnPopup.Glyph.ReleaseHandle;
  FBtnPopup.Glyph.Handle := LoadBitmap(hInstance, 'BTNPOPUP');
  FBtnPopup.NumGlyphs := 2;
  FBtnPopup.OnClick := PopupCalendar;
  FBtnPopup.TabStop := doButtonTabStop in FOptions;

  CreateFormCalendar;
{  Perform (WM_SIZE, SIZE_RESTORED, MAKELONG(width, height));}
end;

{ create the Calendar }
procedure TDateEdit97.Loaded;
begin
  inherited Loaded;
  FColorNormal := Font.Color;
  InitFormCalendar;
end;

procedure TDateEdit97.CreateFormCalendar;
begin
  {Create Form}
{BM}  FrmCalendar := tDtCalPop97.Create (nil);{}
{  FrmCalendar := tDtCalPop97.Create (Self);}
end;

procedure TDateEdit97.InitFormCalendar;
begin
  FrmCalendar.Font := Self.Font;
  FrmCalendar.Calendar97.Font := Self.Font;
  FrmCalendar.aDateParent := Self;
  FrmCalendar.Calendar97.Colors.Assign( ColorCalendar);
  FrmCalendar.Calendar97.DayNames.Assign( DayNames);
  FrmCalendar.Calendar97.MonthNames.Assign( MonthNames);
  FrmCalendar.Calendar97.BeforeNewMonth := FireBeforeNewMonthEvent;
  FrmCalendar.Calendar97.AfterNewMonth := FireAfterNewMonthEvent;
  FrmCalendar.OnClose := ClosePopFct;
  FrmCalendar.Visible := False;
end;

Procedure TDateEdit97.ClosePopFct (Sender: TObject; var Action: TCloseAction);
var
  aFrmCalendar: tDtCalPop97;
begin
  aFrmCalendar := (Sender as tDtCalPop97);
  if not aFrmCalendar.Cancel
    then SetDate (aFrmCalendar.Calendar97.Date);
  ValidateDate(Sender);
{  SetFocus;}
end;

procedure TDateEdit97.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.Style := Params.Style or WS_CLIPCHILDREN;
  case FAlignment of
    taLeftJustify  : Params.Style := Params.Style or (ES_LEFT {or Es_MULTILINE});
    taRightJustify : Params.Style := Params.Style or (ES_RIGHT {or ES_MULTILINE});
    taCenter       : Params.Style := Params.Style or (ES_CENTER {or Es_MULTILINE});
  end;
end;

destructor TDateEdit97.Destroy;
begin
  if Assigned(fBtnClear)
  then begin
    FBtnClear.Free;
    FBtnClear := nil;
  end;
  if Assigned(fBtnValidate)
  then begin
    FBtnValidate.Free;
    FBtnValidate := nil;
  end;
  if Assigned(fBtnPopup)
  then begin
    FBtnPopup.Free;
    FBtnPopup := nil;
  end;

{  ClearValidDates;     {done by FrmCalendar.Free}
  FColorCalendar.Free;
  FDayName.Free;
  FMonthName.Free;
  FrmCalendar.Free;

  FShortCuts.Free;

  inherited Destroy;
end;

(*********************************************
Show (or hide) Popup Calendar when click
*********************************************)
procedure TDateEdit97.PopupCalendar (Sender: TObject);
begin
  Click;
  if Assigned(FrmCalendar)
  then begin
    if not FrmCalendar.Visible
    then begin
      AfterClick(Sender);
      {Transform the number in date to assign it to variables}
      DoDate;
      {check if we need to assign a date, or assign "now"}
      if (Length(Text)=0)
      or (doPopupToday in Options)
      then begin
        FrmCalendar.Calendar97.Date := Now;
      end
      else
        try
          FrmCalendar.Calendar97.Date := FDate
        except
          FrmCalendar.Calendar97.Date := Now;
        end;
      {show modal or not ?}
      if doIsModal in FOptions
        then begin
          FrmCalendar.ShowModal;
        end
        else begin
          FrmCalendar.Show;
          SendMessage(Handle,WM_KILLFOCUS,0,0);
        end;
    end
    else begin
      FrmCalendar.Hide;
    end;
  end;
end;

(*********************************************
Clear the Date field
*********************************************)
procedure TDateEdit97.ClearDate(Sender: TObject);
begin
  Click;
  SetDate(0);
  ValidateDate(Sender);
  SetDateCorrect(True);
  SetFocus;
end;

procedure TDateEdit97.ValidateDate(Sender: TObject);
begin
  if Assigned(FOnValidated)
    then FOnValidated(Sender)
    else SendMessage(TWinControl(Owner).Handle, WM_NEXTDLGCTL, 0, 0);
end;

procedure TDateEdit97.AfterClick(Sender: TObject);
begin
  if Assigned(FOnAfterClick)
    then FOnAfterClick(Self);
end;

procedure TDateEdit97.FireBeforeNewMonthEvent(Sender: TObject; FromDate, TillDate: TDateTime);
begin
 if Assigned(FBeforeNewMonth)
   then FBeforeNewMonth(Sender, FromDate, TillDate);
end;

procedure TDateEdit97.FireAfterNewMonthEvent(Sender: TObject; FromDate, TillDate: TDateTime);
begin
 if Assigned(FAfterNewMonth)
   then FAfterNewMonth(Sender, FromDate, TillDate);
end;

procedure TDateEdit97.CMEnabledChanged(var Message: TMessage);
begin
  if Assigned (FBtnClear) and (doCanClear in FOptions)
    then FBtnClear.Enabled := Enabled;
  if Assigned (FBtnPopup) and (doCanPopup in FOptions)
    then FBtnPopup.Enabled := Enabled;
  if Assigned (FBtnValidate) and (doCanValidate in FOptions)
    then FBtnValidate.Enabled := Enabled;
  inherited;
end;

procedure TDateEdit97.WMSize(var message: TWMSize);
var
  aLeft: integer;
begin
  aLeft := ClientWidth - ClientHeight - 2 ;
  if Assigned(FBtnValidate)
  then begin
    FBtnValidate.Height := ClientHeight;
    FBtnValidate.Width := ClientHeight + 2;
    FBtnValidate.Left := aLeft;
    FBtnValidate.Refresh;
    aLeft := aLeft-FBtnValidate.Width;
  end;
  if Assigned(FBtnPopup)
  then begin
    FBtnPopup.Height := ClientHeight;
    FBtnPopup.Width := ClientHeight + 2;         
    FBtnPopup.Left := aLeft;
    FBtnPopup.Refresh;
    aLeft := aLeft-FBtnPopup.Width;
  end;
  if Assigned(fBtnClear)
  then begin
    FBtnClear.Height := ClientHeight;
    FBtnClear.Width := ClientHeight + 2;
    FBtnClear.Left := aLeft;
    FBtnClear.Refresh;
  end;
end;

procedure TDateEdit97.WMPaint(var Message: TWMPaint);
var Left       : Integer;
    Margins    : TPoint;
    R          : TRect;
    PS         : TPaintStruct;
    OldAlignment,
    AAlignment : TAlignment;
    DC         : HDC;
    S          : string;
    DrawingWidth : Word;
begin
  OldAlignment := FAlignment;
  if Focused and (FAlignment<>taLeftJustify)
  then begin
    Alignment := taLeftJustify;
  end;
  AAlignment := FAlignment;
{ Since edit controls do not handle justification unless multi-line
(and then only poorly) we will draw right and center justify manually
unless the edit has the focus. }
  if FCanvas = nil then
  begin
    FCanvas := TControlCanvas.Create;
    FCanvas.Control := Self;
  end;
  DC := Message.DC;
  if DC = 0 then
    DC := BeginPaint(Handle, PS);
  FCanvas.Handle := DC;
  try
    FCanvas.Font := Font;
    R := ClientRect;
{$IFNDEF WIN32}
    if {not (NewStyleControls and Ctl3D) and }(BorderStyle = bsSingle) then
    begin
      FCanvas.Brush.Color := clWindowFrame;
      FCanvas.FrameRect(R);
      FCanvas.InflateRect(R, -1, -1);
    end;
{$ENDIF}
    FCanvas.Brush.Color := Color;
    if not Enabled then
      FCanvas.Font.Color := clWindowText;
    S := Text;
{      if (csPaintCopy in ControlState) then
    begin}
    case CharCase of
      ecUpperCase: S := AnsiUpperCase(S);
      ecLowerCase: S := AnsiLowerCase(S);
    end;
{    end;}
    if PasswordChar <> #0 then
      FillChar(S[1], Length(S), PasswordChar);
    Margins := GetTextMargins;

    DrawingWidth := ClientWidth;
    if Assigned(FBtnValidate)
    then begin
      DrawingWidth := FBtnValidate.Left
    end;
    if Assigned(FBtnPopup)
    then begin
      DrawingWidth := FBtnPopup.Left;
    end;
    if Assigned(fBtnClear)
    then begin
      DrawingWidth := FBtnClear.Left;
    end;

    case AAlignment of
      taLeftJustify  :
        Left := Margins.X;
      taRightJustify :
        Left := DrawingWidth - FCanvas.TextWidth(S) - Margins.X - 1;
      else
        Left := (DrawingWidth - FCanvas.TextWidth(S)) div 2;
    end;
    FCanvas.TextRect(R, Left, Margins.Y, S);
  finally
    FCanvas.Handle := 0;
    if Message.DC = 0
      then EndPaint(Handle, PS);
    FCanvas.Free;
    FCanvas := nil;
  end;
  if Focused
  then begin
    FAlignment := OldAlignment;
  end;
end;

procedure TDateEdit97.SetKeyRepeat (value: word);
begin
  if Value <> FKeyRepeat
  then begin
    FKeyRepeat := value;
    FrmCalendar.Calendar97.KeyRepeat := FKeyRepeat;
  end;
end;

procedure TDateEdit97.SetOnlyMonth (value: boolean);
begin
  if FOnlyMonth <> value
  then begin
    FOnlyMonth := value;
    FrmCalendar.Calendar97.OnlyMonth := Value;
  end;
end;

procedure TDateEdit97.SetOnlyValidDates (value: boolean);
begin
  if FOnlyValidDates <> value
  then begin
    FOnlyValidDates := value;
    FrmCalendar.Calendar97.OnlyValidDates := Value;
  end;
end;

procedure TDateEdit97.SetDate(value: TDateTime);
begin
  FDate := value;
  SetTextFromDate;
  SetDateCorrect(True);
  Modified := True;
end;

procedure TDateEdit97.SetTextFromDate;
var
  FullYear: boolean;
  CurrYear,
  jj, mm, aa: word;
begin
  if FDate = 0 then
    Text := ''
  else begin
    DecodeDate (Now, CurrYear, mm, jj);
    DecodeDate (FDate, aa, mm, jj);
    if (aa div 100) <> (CurrYear div 100)
    then FullYear := true
    else FullYear := false;
    {if Date century<>Current century, then FullYear=true
    (ie. show century when draw date)}

    if (FullYear and (Length(ShortDateFormat) <= 8))
      then Text := FormatDateTime(PadDate (ShortDateFormat), FDate)
      else Text := FormatDateTime(ShortDateFormat, FDate);
{    if (FullYear and (Length(ShortDateFormat) <= 8))
      then Text := FormatDateTime(PadDate (LongDateFormat), FDate)
      else Text := FormatDateTime(LongDateFormat, FDate);}
  end;
  Refresh;
end;

procedure TDateEdit97.SetDateCorrect (pValue: boolean);
begin
  FDateCorrect := pValue;
  if FDateCorrect
   then
    Font.Color := FColorNormal
  else
    Font.Color := FColorError;
end;

procedure TDateEdit97.DayNameChange(ADayName : TObject);
begin
  FrmCalendar.Calendar97.DayNames.Assign(ADayName as TDayName);
end;

procedure TDateEdit97.MonthNameChange(AMonthName : TObject);
begin
  FrmCalendar.Calendar97.MonthNames.Assign(AMonthName as TMonthName);
end;

function TDateEdit97.GetDayArray: PDayArray;
begin
  FDayArray[1] := DayNames.Monday;
  FDayArray[2] := DayNames.Tuesday;
  FDayArray[3] := DayNames.Wednesday;
  FDayArray[4] := DayNames.Thursday;
  FDayArray[5] := DayNames.Friday;
  FDayArray[6] := DayNames.Saturday;
  FDayArray[7] := DayNames.Sunday;
  Result := @FDayArray;
end;

function TDateEdit97.GetMonthArray: PMonthArray;
begin
  FMonthArray[01] := MonthNames.January;
  FMonthArray[02] := MonthNames.February;
  FMonthArray[03] := MonthNames.March;
  FMonthArray[04] := MonthNames.April;
  FMonthArray[05] := MonthNames.May;
  FMonthArray[06] := MonthNames.June;
  FMonthArray[07] := MonthNames.July;
  FMonthArray[08] := MonthNames.August;
  FMonthArray[09] := MonthNames.September;
  FMonthArray[10] := MonthNames.October;
  FMonthArray[11] := MonthNames.November;
  FMonthArray[12] := MonthNames.December;
  Result := @FMonthArray;
end;

function TDateEdit97.GetValidDates: tStringList;
begin
  Result := FrmCalendar.Calendar97.ValidDates;
end;

function TDateEdit97.GetTextMargins: TPoint;
const
  EM_GETMARGINS = 212;
var
  I          : Integer;
begin
  if BorderStyle = bsNone
  then
    I := 0
  else
  {$IFNDEF WIN32}
    I := 2;
  Result.X := I;  
  {$ELSE}
    I := 1;
  Result.X := SendMessage(Handle, EM_GETMARGINS, 0, 0) and $0000FFFF + I;
  {$ENDIF}
  Result.Y := I;
end;

procedure TDateEdit97.SetBtnStyle (value: tButtonStyle);
begin
  FBtnStyle := value;
  if Assigned (FBtnClear)
    then FBtnClear.Style := FBtnStyle;
  if Assigned (FBtnPopup)
    then FBtnPopup.Style := FBtnStyle;
  if Assigned (FBtnValidate)
    then FBtnValidate.Style := FBtnStyle;
end;

procedure TDateEdit97.SetBtnDateStyle (value: tDateBtnStyle);
begin
  if value<>FBtnDateStyle
  then begin
    FBtnDateStyle := value;
    Case value of
      btsDropDown: begin
                     DeleteObject (FBtnPopup.Glyph.Handle);
                     FBtnPopup.Glyph.ReleaseHandle;
                     FBtnPopup.Glyph.Handle := LoadBitmap(0, pChar(OBM_COMBO));
                     FBtnPopup.NumGlyphs := 1;
                   end;
      btsCalendar: begin
                     DeleteObject (FBtnPopup.Glyph.Handle);
                     FBtnPopup.Glyph.ReleaseHandle;
                     FBtnPopup.Glyph.Handle := LoadBitmap(hInstance, 'BTNPOPUP');
                     FBtnPopup.NumGlyphs := 2;
                   end;
    end;
  end;
end;

procedure TDateEdit97.DoEnter;
begin
  DateToNum;
  { Act like standard edit controls }
  SelectAll;
  inherited DoEnter;
end;

procedure TDateEdit97.DoExit;
begin
  DoDate;
  inherited DoExit;
end;

(*********************************************
Like DoEnter without Selectall
*********************************************)
{procedure TDateEdit97.DoEdit;
begin
  DateToNum;
  inherited DoEnter;
end;}

(*********************************************
Transform number in date and assign values
*********************************************)
procedure TDateEdit97.DoDate;
var
  jj, mm, aa : word;
begin
  if (Length(Text) = 0)
  then begin
    if doEmptyAsToday in FOptions
      then SetDate(Date)
      else SetDate(0);
    SetDateCorrect(True);
  end
  else begin
    {interpret the date, add month and/or year when missing}
    Text := InterpretDate(Text);

    try
      if (Length(Text) < 6) then
        raise EConvertError.Create(ErrDateTooShort);

      {Convert number to date, and correct invalid dates}
      SetDate(TextToDate(NumToDate));
      SetDateCorrect(True);

      if (Length(Text) < 10) {full date is 10 char : dd/mm/yyyy}
      and FUseThreshold
      then begin
        DecodeDate (FDate, aa, mm, jj);
        if (((aa/100)-(aa div 100))*100)<=FCenturyThreshold
        then begin
          aa := aa+100;
          FDate := EncodeDate (aa, mm, jj);
          SetDate(FDate);
          SetDateCorrect(True);
        end
      end;

    except
      if doFocusOnError in FOptions
        then SetFocus;

      SetDateCorrect(False);
    end;
  end;
end;

(*********************************************
Remove a character from a string
*********************************************)
function TDateEdit97.RemoveChr (aChar: Char; aStr: String): String;
var
  i: Integer;
begin
  Result := aStr;
  i := Pos(aChar, Result);
  while i<>0 do
  begin
    Result := Copy (Result, 1, i-1) + Copy (Result, i+1, Length(Result)-i+1);
    i := Pos(aChar, Result);
  end;
end;

(*********************************************
Update local ShortDateFormat with 4digits years
*********************************************)
Function TDateEdit97.PadDate (aStr: String): String;
var
  Count,
  i: integer;
  AddStr: String[3];
begin
  {Pad year to 4 digits}
  Count := 0;
  for i := 1 to Length(aStr) do
    if aStr[i] in ['y', 'Y']
      then inc(Count);
  AddStr := '';
  for i := Count+1 to 4 do
    AddStr := AddStr + 'y';
  i := pos ('Y', UpperCase(aStr));
  Result := Copy (aStr, 1, i-1)+AddStr+Copy (aStr, i, Length(aStr)-i+1);

  {Pad month to 2 digits}
  i := Pos('M', UpperCase(Result));
  if (i = length(Result)) or (UpCase(Result[i + 1]) <> 'M') then
    Insert(Result[i], Result, i);

  {Pad day to 2 digits}
  i := Pos('D', UpperCase(Result));
  if (i = length(Result)) or (UpCase(Result[i + 1]) <> 'D') then
    Insert(Result[i], Result, i);
end;

(*********************************************
Interpret missing fields in date
*********************************************)
function TDateEdit97.InterpretDate (TextDate: string): string;
var
  DateHour: TDateTime;
  StrFormat: String;
begin
  DateHour := SysUtils.Date;
  if doIsMasked in Options
  then
    StrFormat := '%s%s%s'
  else
    StrFormat := '%s'+DateSeparator+'%s'+DateSeparator+'%s';

  case Length(TextDate) of
    2: begin {dd}
         result := Format(StrFormat,[TextDate,FormatDateTime('mm',DateHour),FormatDateTime('yyyy',DateHour)]);
         exit;
       end;
    4: begin {ddmm}
         result := Format(StrFormat,[Copy(TextDate,1,2),Copy(TextDate,3,2),FormatDateTime('yyyy',DateHour)]);
         exit;
       end;
    5: begin {dd/mm}
         result := Format(StrFormat,[Copy(TextDate,1,2),Copy(TextDate,4,2),FormatDateTime('yyyy',DateHour)]);
         exit;
       end;
    6: begin {ddmmyy}
         result := Format(StrFormat,[Copy(TextDate,1,2),Copy(TextDate,3,2),Copy(TextDate,5,2)]);
         exit;
       end;
    8: begin {ddmmyyyy}
         result := Format(StrFormat,[Copy(TextDate,1,2),Copy(TextDate,3,2),Copy(TextDate,5,4)]);
         exit;
       end;
    else
       begin
         result := Text;
       end;
  end;
end;

(*********************************************
Transform number in date
*********************************************)
{procedure TDateEdit97.NumToDate;}
function TDateEdit97.NumToDate : String;
var
  su, sv, sw,
  u, v, w: string;
  lu, lv, lw,
  i, j: integer;
begin
  if (doIsMasked in FOptions)
  and (Pos(DateSeparator, Text) = 0)
  then begin
    i := Pos(DateSeparator, ShortDateFormat);
    j := Pos(DateSeparator, Copy(ShortDateFormat, i + 1, Length(ShortDateFormat)-i)) + i;
    {extract day, month and year from ShortDateFormat}
    su := Copy(ShortDateFormat, 1, i - 1);
    sv := Copy(ShortDateFormat, i + 1, j - i - 1);
    sw := Copy(ShortDateFormat, j + 1, 4);

   {Because we don't know where is year in DateFormat, we need to check it}
   {in all 3 values}
    if (su[1] in ['y', 'Y'])
    then lu := 4
    else lu := 2;
    if (sv[1] in ['y', 'Y'])
    then lv := 4
    else lv := 2;
    if (sw[1] in ['y', 'Y'])
    then lw := 4
    else lw := 2;

    u := Copy(Text, 1, lu);
    v := Copy(Text, lu + 1, lv);
    w := Copy(Text, lu + lv + 1, lw);

{    Text := u + DateSeparator + v + DateSeparator + w;}
    Result := u + DateSeparator + v + DateSeparator + w;
  end;
end;

(*********************************************
Transform date in number
*********************************************)
procedure TDateEdit97.DateToNum;
begin
  if (doIsMasked in FOptions)
  and (Pos(DateSeparator, Text) > 0)
    then Text := FormatDateTime (FNewShortDateFormat, FDate);
end;

(*********************************************
Is the supplied data in the date list?
*********************************************)
function TDateEdit97.DateInList(dt: TDateTime): Boolean;
begin
  Result := FrmCalendar.Calendar97.DateInList (dt);
end;

(*********************************************
Maintain list of valid dates.
*********************************************)
procedure TDateEdit97.AddValidDate(dt: TDateTime);
begin
  FrmCalendar.Calendar97.AddValidDate (dt);
end;

procedure TDateEdit97.ClearValidDates;
begin
  FrmCalendar.Calendar97.ClearValidDates;
end;

procedure TDateEdit97.KeyPress(var Key: Char);
begin
  if (doIsMasked in Options)
    and not (Key in ['0'..'9', #8, #27, #13])
    then Key := #0;

  inherited KeyPress(Key);
end;

(*********************************************
handling procedure (SHORTCUTKEYS)
*********************************************)
procedure TDateEdit97.KeyDown(var Key: Word; Shift: TShiftState);
var
  KeyShortCut: tShortCut;
begin
  KeyShortCut := ShortCut(Key, Shift);
  if KeyShortCut = FShortCuts.BtnClear
  then
    FBtnClear.Click
  else
    if KeyShortCut = FShortCuts.BtnPopup
  then
    FBtnPopup.Click
  else
    if KeyShortCut = FShortCuts.BtnValidate
  then
    FBtnValidate.Click;

  inherited KeyDown(Key, Shift);
end;

procedure TDateEdit97.SetAlignment(value: TAlignment);
begin
  if value <> FAlignment then
  begin
    FAlignment := value;
    RecreateWnd;
  end;
end;

procedure TDateEdit97.SetBevelInner(Value: TPanelBevel);
begin
  if FBevelInner <> Value
  then begin
    FBevelInner := Value;
    FrmCalendar.Calendar97.BevelInner := FBevelInner;
  end;
end;

procedure TDateEdit97.SetBevelOuter(Value: TPanelBevel);
begin
  if FBevelOuter <> Value
  then begin
    FBevelOuter := Value;
    FrmCalendar.Calendar97.BevelOuter := FBevelOuter;
  end;
end;

procedure TDateEdit97.SetStartOfWeek(Value: TDayOfWeek);
begin
  if FStartOfWeek <> Value
  then begin
    FStartOfWeek := Value;
    FrmCalendar.Calendar97.StartOfWeek := FStartOfWeek;
  end;
end;

procedure TDateEdit97.SetStartOfWeekEnd(Value: TDayOfWeek);
begin
  if FStartOfWeekEnd <> Value
  then begin
    FStartOfWeekEnd := Value;
    FrmCalendar.Calendar97.StartOfWeekEnd := FStartOfWeekEnd;
  end;
end;

procedure TDateEdit97.SetWeekEndDays(Value: byte);
begin
  if FWeekEndDays <> Value
  then begin
    FWeekEndDays := Value;
    FrmCalendar.Calendar97.WeekEndDays := FWeekEndDays;
  end;
end;

(*********************************************
Set Options
*********************************************)
procedure TDateEdit97.SetOptions (value: tDateOptions);
begin
  {BtnTabStop}
  if (doButtonTabStop in Value)<>(doButtonTabStop in FOptions)
  then begin
    if Assigned (FBtnClear)
      then FBtnClear.TabStop := doButtonTabStop in Value;
    if Assigned (FBtnPopup)
      then FBtnPopup.TabStop := doButtonTabStop in Value;
    if Assigned (FBtnValidate)
      then FBtnValidate.TabStop := doButtonTabStop in Value;
  end;
  {CanPopup}
  if (doCanPopup in Value)<>(doCanPopup in FOptions)
  then begin
    FBtnPopup.Enabled := doCanPopup in Value;
  end;
  {CanClear}
  if (doCanClear in Value)<>(doCanClear in FOptions)
  then begin
    if doCanClear in Value
    then begin
      if not Assigned (FBtnClear)
        then FBtnClear := tBitBtn.Create (Self);
      FBtnClear.Visible := True;
      FBtnClear.Parent := Self;
      DeleteObject (FBtnClear.Glyph.Handle);
      FBtnClear.Glyph.ReleaseHandle;
      FBtnClear.Glyph.Handle := LoadBitmap(hInstance, 'BTNCLEAR');
      FBtnClear.NumGlyphs := 2;
      FBtnClear.OnClick := ClearDate;
      FBtnClear.TabStop := doButtonTabStop in Value;
    end
    else begin
      if Assigned(fBtnClear)
      then begin
        FBtnClear.Free;
        FBtnClear := nil;
      end;
    end;
  end;
  {CanValidate}
  if (doCanValidate in Value)<>(doCanValidate in FOptions)
  then begin
    if doCanValidate in Value
    then begin
      if not Assigned (FBtnValidate)
        then FBtnValidate := tBitBtn.Create (Self);
      FBtnValidate.Visible := True;
      FBtnValidate.Parent := Self;
      DeleteObject (FBtnValidate.Glyph.Handle);
      FBtnValidate.Glyph.ReleaseHandle;
      FBtnValidate.Glyph.Handle := LoadBitmap(hInstance, 'BTNVALIDATE');
      FBtnValidate.NumGlyphs := 2;
      FBtnValidate.OnClick := ValidateDate;
      FBtnValidate.TabStop := doButtonTabStop in Value;
    end
    else begin
      if Assigned(FBtnValidate)
      then begin
        FBtnValidate.Free;
        FBtnValidate := nil;
      end;
    end;
  end;

  {ShowCancel}
  if (doShowCancel in Value)<>(doShowCancel in FOptions)
  then begin
    FrmCalendar.Calendar97.ShowCancel := (doShowCancel in Value);
  end;

  {ShowNumWeek}
  if (doShowNumWeek in Value)<>(doShowNumWeek in FOptions)
  then begin
    FrmCalendar.Calendar97.ShowNumWeek := (doShowNumWeek in Value);
  end;

  {ShowToday}
  if (doShowToday in Value)<>(doShowToday in FOptions)
  then begin
    FrmCalendar.Calendar97.ShowToday := (doShowToday in Value);
  end;

  FOptions := Value;
  Perform (WM_SIZE, SIZE_RESTORED, MAKELONG(width, height));
end;

end.


procedure TDateEdit97.WMPaint(var Message: TWMPaint);
var Left       : Integer;
    Margins    : TPoint;
    R          : TRect;
    PS         : TPaintStruct;
    OldAlignment,
    AAlignment : TAlignment;
    DC         : HDC;
    S          : string;
    DrawingWidth : Word;
begin
  OldAlignment := FAlignment;
  if Focused and (FAlignment<>taLeftJustify)
  then begin
    Alignment := taLeftJustify;
  end;
  AAlignment := FAlignment;
{ Since edit controls do not handle justification unless multi-line
(and then only poorly) we will draw right and center justify manually
unless the edit has the focus. }
  if FCanvas = nil then
  begin
    FCanvas := TControlCanvas.Create;
    FCanvas.Control := Self;
  end;
  DC := Message.DC;
  if DC = 0 then
    DC := BeginPaint(Handle, PS);
  FCanvas.Handle := DC;
  try
    FCanvas.Font := Font;
    with FCanvas do
    begin
      R := ClientRect;
{$IFNDEF WIN32}
      if {not (NewStyleControls and Ctl3D) and }(BorderStyle = bsSingle) then
      begin
        Brush.Color := clWindowFrame;
        FrameRect(R);
        InflateRect(R, -1, -1);
      end;
{$ENDIF}
      Brush.Color := Color;
      if not Enabled then
        Font.Color := clWindowText;
      S := Text;
{      if (csPaintCopy in ControlState) then
      begin}
      case CharCase of
        ecUpperCase: S := AnsiUpperCase(S);
        ecLowerCase: S := AnsiLowerCase(S);
      end;
{    end;}
      if PasswordChar <> #0 then
        FillChar(S[1], Length(S), PasswordChar);
      Margins := GetTextMargins;

      DrawingWidth := clientWidth;
      if Assigned(FBtnValidate)
      then begin
        DrawingWidth := FBtnValidate.Left
      end;
      if Assigned(FBtnPopup)
      then begin
        DrawingWidth := FBtnPopup.Left;
      end;
      if Assigned(fBtnClear)
      then begin
        DrawingWidth := FBtnClear.Left;
      end;

      case AAlignment of
        taLeftJustify  :
          Left := Margins.X;
        taRightJustify :
          Left := DrawingWidth - TextWidth(S) - Margins.X - 1;
        else
          Left := (DrawingWidth - TextWidth(S)) div 2;
      end;
      TextRect(R, Left, Margins.Y, S);
    end;
  finally
    FCanvas.Handle := 0;
    if Message.DC = 0
      then EndPaint(Handle, PS);
    FCanvas.Free;
    FCanvas := nil;
  end;
  if Focused
  then begin
    FAlignment := OldAlignment;
  end;
end;


